/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.jdbc;

import java.util.List;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jdbc.core.convert.JdbcCustomConversions;
import org.springframework.data.jdbc.repository.QueryMappingConfiguration;
import org.springframework.data.jdbc.repository.config.AbstractJdbcConfiguration;
import org.springframework.data.jdbc.repository.config.DefaultQueryMappingConfiguration;

import xyz.zhouxy.plusone.jdbc.converter.EnumToOrdinalConverter;

/**
 * JDBC 配置
 *
 * <p>
 * 配置了类型转换器
 * </p>
 *
 * <p>
 * 这里声明了 {@link DefaultQueryMappingConfiguration} 的实例为 Spring bean。
 * 在其它配置类中注入该 bean，就可以向其中添加
 * {@link org.springframework.jdbc.core.RowMapper}
 * 供 Spring Data JDBC 查询时使用。如下所示：
 *
 * <pre>
 * {@code @Configuration}
 * class CustomConfig {
 *     public CustomConfig(DefaultQueryMappingConfiguration rowMappers) {
 *         rowMappers.registerRowMapper(Person.class, new PersonRowMapper());
 *     }
 * }
 * </pre>
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Configuration
public class JdbcConfig extends AbstractJdbcConfiguration {

    @Override
    public JdbcCustomConversions jdbcCustomConversions() {
        return new JdbcCustomConversions(List.of(EnumToOrdinalConverter.INSTANCE));
    }

    @Bean
    QueryMappingConfiguration rowMappers() {
        return new DefaultQueryMappingConfiguration();
    }
}
