/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.jdbc;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import xyz.zhouxy.plusone.domain.Entity;

public abstract class JdbcEntityDaoSupport<T extends Entity<ID>, ID extends Serializable>
        extends PlusoneJdbcDaoSupport {

    protected RowMapper<T> rowMapper;
    protected ResultSetExtractor<Optional<T>> resultSetExtractor;

    protected JdbcEntityDaoSupport(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.rowMapper = (ResultSet rs, int rowNum) -> mapRow(rs);
        this.resultSetExtractor = (ResultSet rs) -> rs.next() ? Optional.of(mapRow(rs)) : Optional.empty();
    }

    protected final Optional<T> queryForObject(String sql) {
        return queryForObject(sql, this.resultSetExtractor);
    }

    protected final Optional<T> queryForObject(String sql, SqlParameterSource paramSource) {
        return queryForObject(sql, paramSource, this.resultSetExtractor);
    }

    protected final Optional<T> queryForObject(String sql, String paramName, Object value) {
        return queryForObject(sql, new MapSqlParameterSource(paramName, value), this.resultSetExtractor);
    }

    protected final List<T> queryForList(String sql) {
        return queryForList(sql, this.rowMapper);
    }

    protected final List<T> queryForList(String sql, SqlParameterSource parameterSource) {
        return queryForList(sql, parameterSource, this.rowMapper);
    }

    protected final List<T> queryForList(String sql, String paramName, Object value) {
        return queryForList(sql, new MapSqlParameterSource(paramName, value), this.rowMapper);
    }

    protected final Stream<T> queryForStream(String sql, SqlParameterSource parameterSource) {
        return queryForStream(sql, parameterSource, this.rowMapper);
    }

    protected final Stream<T> queryForStream(String sql, String paramName, Object value) {
        return queryForStream(sql, new MapSqlParameterSource(paramName, value), this.rowMapper);
    }

    protected abstract T mapRow(ResultSet rs) throws SQLException;

    protected void setRowMapper(@Nonnull RowMapper<T> rowMapper) {
        this.rowMapper = rowMapper;
    }

    protected void setResultSetExtractor(@Nonnull ResultSetExtractor<Optional<T>> resultSetExtractor) {
        this.resultSetExtractor = resultSetExtractor;
    }
}
