/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.jdbc.converter;

import org.springframework.core.convert.converter.Converter;
import org.springframework.data.convert.ReadingConverter;

/**
 * 序号枚举转换器
 * <p>
 * 向 {@link org.springframework.data.jdbc.core.convert.JdbcCustomConversions}
 * 中添加类型转换器，
 * 如：
 *
 * <pre>
 * return new JdbcCustomConversions(List.of(new OrdinalToEnumConverter<SystemStatus>()));
 * </pre>
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 *
 * @see Converter
 * @see org.springframework.data.jdbc.core.convert.JdbcCustomConversions
 */
@ReadingConverter
public class OrdinalToEnumConverter<E extends Enum<E>> implements Converter<Integer, Enum<E>> {

    private final Class<E> type;
    private final E[] constants;

    public OrdinalToEnumConverter(Class<E> type) {
        this.type = type;
        this.constants = type.getEnumConstants();
    }

    @Override
    public Enum<E> convert(Integer ordinal) {
        try {
            return constants[ordinal];
        } catch (ArrayIndexOutOfBoundsException exception) {
            throw new IllegalArgumentException(
                    String.format("Cannot convert %d to %s by ordinal value.", ordinal, type.getSimpleName()),
                    exception);
        }
    }
}
