/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.mail;

import lombok.extern.slf4j.Slf4j;
import org.springframework.mail.MailException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;

import javax.mail.MessagingException;

/**
 * 邮件服务。<b>不能发送 HTML 邮件。</b>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Slf4j
public class SimpleMailService implements MailService {

    private final JavaMailSender mailSender;
    private final MailMessageFactory mailMessageFactory;

    public SimpleMailService(JavaMailSender mailSender, MailMessageFactory mailMessageFactory) {
        this.mailSender = mailSender;
        this.mailMessageFactory = mailMessageFactory;
    }

    /**
     * 发送文本邮件
     */
    @Override
    public void sendSimpleMail(String to, String subject, String content) {
        SimpleMailMessage message = mailMessageFactory.getSimpleMailMessage(to, subject, content);
        try {
            mailSender.send(message);
            log.debug("简单邮件已经发送。");
        } catch (MailException e) {
            log.error("发送简单邮件时发生异常！", e);
            throw e;
        }
    }

    @Override
    public void sendHtmlMail(String to, String subject, String content) throws MessagingException {
        throw new UnsupportedOperationException("暂不支持");
    }

    @Override
    public void sendCodeMail(String code, String to) {
        SimpleMailMessage message = mailMessageFactory.getCodeMailMessage(code, to);
        try {
            mailSender.send(message);
            log.debug("简单邮件已经发送。");
        } catch (MailException e) {
            log.error("发送简单邮件时发生异常！", e);
            throw e;
        }
    }
}
