/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.web.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;
import org.springframework.format.FormatterRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * Spring MVC 配置。
 *
 * <p>
 * 将与前后端交互的数据中的枚举转换为其整数值
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Configuration
public class WebEnumMapConfig implements WebMvcConfigurer {
    @Override
    public void addFormatters(FormatterRegistry registry) {
        registry.addConverterFactory(new IntToEnumConverterFactory());
        registry.addConverterFactory(new StringToEnumConverterFactory());
    }
}


class IntToEnumConverterFactory implements ConverterFactory<Integer, Enum<?>> {
    @Override
    public <T extends Enum<?>> Converter<Integer, T> getConverter(Class<T> targetType) {
        return (Integer source) -> {
            try {
                T[] values = targetType.getEnumConstants();
                return values[source];
            } catch (IndexOutOfBoundsException e) {
                throw new EnumConstantNotPresentException(targetType, Integer.toString(source));
            }
        };
    }
}

class StringToEnumConverterFactory implements ConverterFactory<String, Enum<?>> {
    @Override
    public <T extends Enum<?>> Converter<String, T> getConverter(Class<T> targetType) {
        return (String source) -> {
            int index = Integer.parseInt(source);
            try {
                T[] values = targetType.getEnumConstants();
                return values[index];
            } catch (IndexOutOfBoundsException e) {
                throw new EnumConstantNotPresentException(targetType, Integer.toString(index));
            }
        };
    }
}
