/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.system.application.common.util;

import javax.annotation.Nullable;

import xyz.zhouxy.plusone.system.application.common.exception.UnsupportedPrincipalTypeException;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Principal;
import xyz.zhouxy.plusone.system.domain.model.account.Username;

/**
 * 根据字面值，判断并生成 {@link Principal} 值对象。
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @see Principal
 * @see Username
 * @see Email
 * @see MobilePhone
 * @see UnsupportedPrincipalTypeException
 */
public class PrincipalUtil {

    private PrincipalUtil() {
        throw new IllegalStateException("Utility class");
    }

    public static PrincipalType getPrincipalType(@Nullable String principal) {
        if (principal == null) {
            throw new IllegalArgumentException("principal 不能为空");
        }
        PrincipalType[] principalTypes = PrincipalType.values();
        for (var principalType : principalTypes) {
            if (principalType.getRegex().matcher(principal).matches()) {
                return principalType;
            }
        }
        throw new UnsupportedPrincipalTypeException();
    }

    public static Principal getPrincipal(@Nullable String principal) {
        PrincipalType principalType = getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            return Email.of(principal);
        }
        if (principalType == PrincipalType.MOBILE_PHONE) {
            return MobilePhone.of(principal);
        }
        if (principalType == PrincipalType.USERNAME) {
            return Username.of(principal);
        }
        throw new UnsupportedPrincipalTypeException();
    }

    public static Principal getEmailOrMobilePhone(@Nullable String principal) {
        PrincipalType principalType = getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            return Email.of(principal);
        }
        if (principalType == PrincipalType.MOBILE_PHONE) {
            return MobilePhone.of(principal);
        }
        throw new UnsupportedPrincipalTypeException("输入邮箱地址或手机号");
    }
}
