/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.system.application.service;

import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.validation.Valid;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import xyz.zhouxy.plusone.constant.EntityStatus;
import xyz.zhouxy.plusone.domain.IWithOrderNumber;
import xyz.zhouxy.plusone.exception.DataNotExistException;
import xyz.zhouxy.plusone.system.application.exception.UnsupportedMenuTypeException;
import xyz.zhouxy.plusone.system.application.query.result.MenuViewObject;
import xyz.zhouxy.plusone.system.application.service.command.CreateMenuCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateMenuCommand;
import xyz.zhouxy.plusone.system.domain.model.menu.Menu;
import xyz.zhouxy.plusone.system.domain.model.menu.MenuConstructor;
import xyz.zhouxy.plusone.system.domain.model.menu.MenuRepository;
import xyz.zhouxy.plusone.system.domain.service.MenuService;

/**
 * 菜单管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class MenuManagementService {

    private final MenuService menuService;
    private final MenuRepository menuRepository;

    MenuManagementService(MenuService roleRepository, MenuRepository menuRepository) {
        this.menuService = roleRepository;
        this.menuRepository = menuRepository;
    }

    // ==================== create ====================
    public void createMenu(@Valid CreateMenuCommand command) {
        Menu menuToInsert;
        switch (command.getMenuType()) {
            case MENU_LIST:
                menuToInsert = createMenuList(command);
                break;
            case MENU_ITEM:
                menuToInsert = createMenuItem(command);
                break;
            default:
                throw new UnsupportedMenuTypeException();
        }
        menuRepository.save(menuToInsert);
    }

    private Menu createMenuList(CreateMenuCommand command) {
        return MenuConstructor.newMenuList(
                command.getParentId(),
                command.getPath(),
                command.getName(),
                command.getTitle(),
                command.getIcon(),
                command.getHidden(),
                command.getOrderNumber(),
                EntityStatus.of(command.getStatus()),
                command.getRemarks());
    }

    private Menu createMenuItem(CreateMenuCommand command) {
        return MenuConstructor.newMenuItem(
                command.getParentId(),
                command.getPath(),
                command.getName(),
                command.getTitle(),
                command.getIcon(),
                command.getHidden(),
                command.getOrderNumber(),
                EntityStatus.of(command.getStatus()),
                command.getComponent(),
                command.getResource(),
                command.getCache(),
                command.getRemarks());
    }

    // ==================== delete ====================
    public void deleteMenu(Long id) {
        Menu menuToDelete = menuRepository.find(id)
                .orElseThrow(DataNotExistException::new);
        menuRepository.delete(menuToDelete);
    }

    // ==================== update ====================
    public void updateMenu(Long id, @Valid UpdateMenuCommand command) {
        Assert.isTrue(Objects.equals(id, command.getId()), "id 不匹配");
        Menu menuToUpdate = menuRepository.find(command.getId())
                .orElseThrow(DataNotExistException::new);
        menuToUpdate.updateMenuInfo(
                command.getMenuType(),
                command.getParentId(),
                command.getPath(),
                command.getName(),
                command.getTitle(),
                command.getIcon(),
                command.getHidden(),
                command.getOrderNumber(),
               EntityStatus.of(command.getStatus()),
                command.getComponent(),
                command.getResource(),
                command.getCache(),
                command.getRemarks());
        menuRepository.save(menuToUpdate);
    }

    // ==================== query ====================
    @Transactional(propagation = Propagation.SUPPORTS)
    public MenuViewObject findById(Long id) {
        var menu = menuRepository.find(id);
        return MenuViewObject.of(menu.orElseThrow(DataNotExistException::new));
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<MenuViewObject> queryByAccountId(Long accountId) {
        var menus = menuService.queryAllMenuListByAccountId(accountId);
        var menuViewObjects = menus.stream().map(MenuViewObject::of).toList();
        return buildMenuTree(menuViewObjects);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<MenuViewObject> queryByRoleId(Long roleId) {
        var menus = menuRepository.queryByRoleId(roleId);
        var menuViewObjects = menus.stream().map(MenuViewObject::of).toList();
        return buildMenuTree(menuViewObjects);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<MenuViewObject> buildMenuTree(List<MenuViewObject> menus) {
        List<MenuViewObject> rootMenus = menus
                .stream()
                .filter(menu -> Objects.equals(menu.getParentId(), 0L))
                .toList();

        Map<Long, MenuViewObject> allMenus = new HashMap<>();
        for (var item : menus) {
            allMenus.put(item.getId(), item);
        }

        for (MenuViewObject menu : menus) {
            long parentId = menu.getParentId();
            while (parentId != 0 && !allMenus.containsKey(parentId)) {
                MenuViewObject parent = findById(parentId);
                if (parent == null) {
                    break;
                }
                allMenus.put(parent.getId(), parent);
                parentId = parent.getParentId();
            }
        }

        for (var menu : allMenus.values()) {
            var parent = allMenus.getOrDefault(menu.getParentId(), null);
            if (parent != null) {
                parent.addChild(menu);
            }
        }

        return rootMenus
                .stream()
                .sorted(Comparator.comparing(IWithOrderNumber::getOrderNumber))
                .toList();
    }
}
